from enum import Enum
from logger import Logger
from typing import Dict, List, Any, Union
import bottle
import json
import os
import time


"""
Формат тест кейса таков:
{
  "<url-путь>": [
    {
       # строка(или null) ответа от сервера
       "response": str,  
       
       # метаданные ответа(могут отсутствовать и тогда будут использованы значения по умолчанию)
       "responsemeta": {
         # строковое перечисление типа возвращаемых данных(по умолчанию dtext)
         "datatype": Enum(str) {dtext, djson, dxml},
         
         # строковое перечисление источника данных для ответа на запрос(по умолчанию stext)
         # если значение sfile, то считаем что данные ответа лежат в файле по пути: 
         #    "/linuxcash/cash/servicestub/services/<имя_сервиса>/ + <значение_поля_response>" 
         "source": Enum(str) {stext, sfile}
       },
       
       # возвращаемая ошибка http(если присутствует, то сервис вернёт её на запрос)
       "error": {
         # код ошибки
         "code": int
       },
       
       # счётчик ответов на запрос. Если отсутствует, то ответ будет возвращаться всегда
       "count": int
       
       # таймаут перед ответом сервиса. Может отсутствовать
       "timeout": float
    }
  ]
}

"""


class EDataType(str, Enum):
    dtext = 'text'
    djson = 'json'
    dxml = 'xml'


class ESource(str, Enum):
    stext = 'text'
    sfile = 'file'


class ResponseMeta:
    datatype: Enum
    source: Enum

    def __init__(self, datatype=None, source=None, **kwargs):
        self.datatype = EDataType.dtext if datatype is None else datatype
        self.source = ESource.stext if source is None else source


class HttpError:
    code: int

    def __init__(self, code=200, **kwargs):
        self.code = code


class Case:
    response: Union[str, None]
    responsemete: Union[ResponseMeta, None]
    error: Union[HttpError, None]
    count: Union[int, None]
    timeout: Union[int, None]

    def __init__(self, response=None, responsemeta=None, error=None, count=None, timeout=None, **kwargs):
        self.response = response
        self.responsemeta = None if responsemeta is None else ResponseMeta(**responsemeta)
        self.error = None if error is None else HttpError(**error)
        self.count = count
        self.timeout = timeout

    def createResponse(self, servicePath: str):
        if self.error is not None:
            return bottle.HTTPError(self.error.code)

        # первое - это получение данных
        content = self.response
        if self.responsemeta.source == ESource.sfile:
            with open(os.path.join(servicePath, self.response)) as f:
                content = f.read()

        # второе - это формирование ответа
        if self.responsemeta.datatype == EDataType.djson:
            content = json.dumps(content)

        # TODO: параметризировать status и headers
        return bottle.HTTPResponse(body=content)


class BasicService(object):
    log: Logger
    servicePath: str
    cases: Dict[str, List[Case]]

    def __init__(self):
        self.log = Logger()
        self.servicePath = '/'
        self.cases = {}

    def _loadCases(self, filepath):
        try:
            with open(filepath, 'r') as casesFile:
                cases = json.load(casesFile)
            for path in cases:
                self.cases[path] = [Case(**case) for case in cases[path]]
        except Exception as e:
            self.log.error(str(e))

    def _createResponse(self, methodName: str):
        if methodName not in self.cases:
            raise bottle.HTTPError(400, 'Метод "%s" не найден' % methodName)

        if not len(self.cases[methodName]):
            self.log.warn('В кейсах не осталось значений')
            raise bottle.HTTPError(500, 'В кейсах не осталось значений')

        case = self.cases[methodName][0]

        if case.timeout is not None:
            time.sleep(case.timeout)

        response = case.createResponse(self.servicePath)

        # уменьшить счётчик кейса, если он есть
        if case.count is not None:
            case.count -= 1
            if case.count == 0:
                del self.cases[methodName][0]

        if response is bottle.HTTPError:
            raise response
        return response
